/*************************************************************************************
#
#	Command.h
#	
#	The command section accomplishes two things for the game -- first, providing a
#	standard clock to which all actions can be synchonized, and second, grabbing the state
#	of the keyboard to pass along to those game parts that need it.
#
#	This code implements a VBL take that goes out and grabs the state of the keyboard once
#	per frame of animation.  To minimize the hit, all this does is save this command state
#	off into a buffer and then exit.
#
#	We create a buffer for two seconds worth of commands, although in practice we want to
#	pull commands from the buffer much faster than this.
#
#	Eventually, this code will also probably need to deal with networking tasks.
#
#	Author: Timothy Carroll
#	Apple Developer Technical Support
#	timc@apple.com
#
#	Modification History: 
#
#	8/15/96		TMC 	Initial Release			 
#
#	Copyright  1996 Apple Computer, Inc., All Rights Reserved
#
#
#	You may incorporate this sample code into your applications without
#	restriction, though the sample code has been provided "AS IS" and the
#	responsibility for its operation is 100% yours.  However, what you are
#	not permitted to do is to redistribute the source as "DSC Sample Code"
#	after having made changes. If you're going to re-distribute the source,
#	we require that you make it clear in the source that the code was
#	descended from Apple Sample Code, but that you've made changes.
#
*************************************************************************************/
#ifndef _COMMAND_
#define _COMMAND_

#pragma once

#include <Events.h>

#if PRAGMA_ALIGN_SUPPORTED
#pragma options align=power
#endif

// This structure holds all of the data for one "frame" of the game.  For the moment,
// we're just sending back a copy of the KeyMap, but in the future we might grab
// the state of the mouse, network events, and so on.

struct TCommand {
	KeyMap		keys;
};


// When you initialize the game, pass in one of these constants to get the desired maximum
// frame rate.  Note that this is not necessarily the visual frame rate, but the rate
// at which the computer will generate TCommand structures.  In an ideal world, the
// graphics frame rate will keep up with this frame rate.

enum {
	k60Frames = 1,
	k30Frames = 2,
	k20Frames = 3,
	k15Frames = 4,
	k12Frames = 5
};

// Call this routine with the constant to initialize the VBL task.  This must be called
// before any other routines are used.
OSErr InitializeCommands (SInt16 framesDesired);

// Call this to dispose of the VBL task, like when you finish the game.
OSErr ExitCommands (void);

// When starting the game, you should call this to clear all commands from the buffer.
// This routine should only be called if the VBL task is not running, see below.
void FlushCommandBuffer (void);

// Calling this routine starts the timer on the game.  Since this timer is used to control
// the game timing, it should be one of the last things called before executing the game
// loop.
void StartAcceptingCommands (void);

// Similarly, this stops the VBL task so that no commands get written to the buffer.
// Call this whenever the game ends or when the user pauses the game.

void StopAcceptingCommands (void);

// This routine returns TRUE whenever there is a new command waiting in the buffer.
Boolean IsCommandWaiting (void);

// Calling this routine retrieves the next command from the buffer.  If there is no
// command waiting, it returns false and leaves outCommand unchanged.
Boolean RetrieveCommand (TCommand *outCommand);


#if PRAGMA_ALIGN_SUPPORTED
#pragma options align=reset
#endif


#endif /* _COMMAND_ */
