/*************************************************************************************
#
#	TGraphic.cp
#	
#
#
#	The TGraphic class encapsulates custom blitting of a graphic to an offscreen GWorld.  The
#	current blitters are based on the Encoded Sprite from "Tips of the Mac Gaming Gurus", with
#	a number of additions and refinements.  For example, this class handles hit testing for a 
#	single point, hit testing any two graphics against each other, and using the sprite data
#	as a mask for copying from a "background" GWorld into the "foreground" GWorld.  This class
#	also handles some alignment issues for the 604, which can slow down the normal TGraphic
#	class.
#	
#	Assumptions:  Currently, this class assumes that the color table used to encode the sprites
#	is loaded and being used for both GWorlds.  The GWorlds must have a bit depth of 8 -- no
#	other bit depth will work properly.
#	
#	All TGraphics share the same destinations, which are set up with the "PrepareDrawing" and
#	"EndDrawing" commands.
#
#	Note that this class isn't currently intended to be subclassed, so all of the methods are 
#	non-virtual.  This might change if at some point we determine we need to create a subclass.
#
#	Author: Timothy Carroll
#	Apple Developer Technical Support
#	timc@apple.com
#
#	Modification History: 
#
#	6/1/96		TMC 	Initial Release
#
#	Copyright  1996 Apple Computer, Inc., All Rights Reserved
#
#
#	You may incorporate this sample code into your applications without
#	restriction, though the sample code has been provided "AS IS" and the
#	responsibility for its operation is 100% yours.  However, what you are
#	not permitted to do is to redistribute the source as "DSC Sample Code"
#	after having made changes. If you're going to re-distribute the source,
#	we require that you make it clear in the source that the code was
#	descended from Apple Sample Code, but that you've made changes.
#
*************************************************************************************/


#include <Memory.h>
#include <Resources.h>
#include <Icons.h>
#include "TGraphic.h"
#include "Scaling.h"


/****************************************************************************************************

	Internal Declarations
	
	We do a number of internal declarations that are used by the TGraphic class.  First, we 
	define a number of variables to hold the drawing data -- specifically the PixMaps and clipping
	rectangles, along with all the other associated data.  We keep it here to make all drawing
	very speedy.
	
	In addition, we define the actual constants used by the sprite encoder and decoded here.
	
	Each TGraphic consists of a number of drawing commands.  A drawing command is a 32 byte unsigned
	integer with an 8-bit operation and a 24-bit value used by that operation.  Each command is
	followed by zero or more bytes of actual data.  Currently, all of the sprite data is also aligned
	to a long-boundary, we might see a speed improvement on PowerPC if we align the drawing data to
	an 8-byte boundary.
	
****************************************************************************************************/


// We'll use this to align to the next available long, if we are misaligned.  We can change this
// macro to align to any length.
#define ALIGN_TO_NEXT_LONG(x)  ( ((unsigned long) x + 3) & ~3)


/****************************************************************************************************
	Sprite Tokens
	
	kEndShapeToken ignores the tokenData and notes the end of the graphic.
	
	kLineStartToken precedes each line in the object.  TokenData holds the offset to the next line,
    which is useful for vertical clipping.
    
    kDrawPixelsToken draws a sequence of pixels.  A draw command is the only thing currently followed
    by data -- all others are EXACTLY 4 bytes -- just the command.  The draw command is followed by
    a number of bytes equal to the TokenData value, and then padded to the next available 4 byte
    boundary.
    
    KSkipPixels just skips across a number of pixels in the image.  No image data follows it.
****************************************************************************************************/

#define kEndShapeToken		0L
#define kLineStartToken		1L
#define kDrawPixelsToken	2L
#define kHitTestPixelsToken 2L
#define kSkipPixelsToken	3L
#define kClearColorIndex	0
#define kMaskColorIndex		255

// This is the standard header we use when loading a TGraphic resource or creating a new one.

struct TGraphicResHeader
{
	UInt32 version;
	SInt16 depth;
	UInt16 flags;
	Rect   bounds;
	UInt32 imageSize;
	UInt32 maskSize;
};

#define kTGraphicResourceType 'Gfic'

// Allocations for the encoder
#define kPreferredDepth 8

typedef TGraphicResHeader **TGraphicResHeaderHandle;
/****************************************************************************************************
	Internal Allocation
	
	We use reference counting to minimize the number of copies of a particular image that need to be
	loaded, useful when you might have 50-100 sprites using the same image.  We keep track of this
	with a handle to the number of TGraphic objects, sorted by the resource ID of the TGraphic.  These
	routines are almost identical to those found in the TGraphic class.
	
****************************************************************************************************/

static Handle gTGraphicList;
static unsigned long gNumberGraphics;

static OSErr InsertGraphicIntoList (TGraphic *theGraphic, UInt32 index);
static OSErr DeleteGraphicFromList (UInt32 index);
static OSErr SearchGraphicList (SInt16 resID, Boolean *found, UInt32 *index);

/****************************************************************************************************
	InsertGraphicIntoList
	
	This routine creates the handle if necessary, and otherwise inserts the TGraphic
	into the list at the index location given.
	
	Assumptions:
		theGraphic must be a legitimate TGraphic -- no NULL parameters.
		index must be no more than 1 larger than gNumberGraphics.
****************************************************************************************************/

OSErr InsertGraphicIntoList (TGraphic *theGraphic, UInt32 index)
{
	OSErr theErr = noErr;

#if qDebugging
	if (index > gNumberGraphics)
		SIGNAL_ERROR ("\pAttempting to insert Graphic at invalid index")
	if (theGraphic == NULL)
		SIGNAL_ERROR ("\pAttempting to insert a null Graphic into the list")	
	
#endif
	
	if (gTGraphicList == NULL)
	{
		gNumberGraphics = 1;
		
		gTGraphicList = NewHandleClear (sizeof (TGraphic *));
		theErr = MemError();
		
		FAIL_OSERR (theErr, "\pCouldn't allocate a new handle of information")
		FAIL_NIL (gTGraphicList, "\pCouldn't allocate a new handle of information")
		
	}
	else
	{
		gNumberGraphics++;
		SetHandleSize(gTGraphicList,gNumberGraphics*sizeof(TGraphic *));
		
		theErr = MemError();
		FAIL_OSERR (theErr, "\pCouldn't resize the list handle")
		FAIL_NIL (gTGraphicList, "\pCouldn't resize the list handle")
		
		// Shift the data to make room
		BlockMoveData(	(*(TGraphic ***)gTGraphicList)+index,
						(*(TGraphic ***)gTGraphicList)+index+1,
						(gNumberGraphics - index-1) * sizeof(TGraphic *));
	}
	
	// finally, set the new object in place
			*((*(TGraphic ***) gTGraphicList)+index) = theGraphic;

	return noErr;
	
	error:
	
	if (theErr == noErr)
		theErr = paramErr;
	return paramErr;
}


/****************************************************************************************************
	DeleteGraphicFromList
	
	This routine removes the collection from the list and destroys it.  If there are no more items
	in the list then we dispose of the handle.
	
	Assumptions:
	index must be within the list.
****************************************************************************************************/
OSErr DeleteGraphicFromList (UInt32 index)
{
	OSErr theErr;
	
	gNumberGraphics--;
	//	slide remaining elements up
	BlockMoveData(	(*(TGraphic ***)gTGraphicList)+index+1,
					(*(TGraphic ***)gTGraphicList)+index,
					(gNumberGraphics - index) * sizeof(TGraphic *));

	//	cut back the storage and dispose of the handle if we have no items left

	
	if (gNumberGraphics> 0)
	{
		SetHandleSize((Handle) gTGraphicList,gNumberGraphics*sizeof(TGraphic *));
		theErr = MemError();
		FAIL_OSERR (theErr, "\pCouldn't resize the list handle")
		FAIL_NIL (gTGraphicList, "\pCouldn't resize the list handle")
	}
	
	else
	{
		DisposeHandle (gTGraphicList);
		gTGraphicList = NULL;
	}
	
	
	return noErr;
	error:
	
	if (theErr == noErr)
		theErr = paramErr;
	return paramErr;
}


/****************************************************************************************************
	SearchGraphicList
	
	This routine searches through the list and attempts to find an existing TGraphic with
	that resID.  If it finds one with that resID, then it returns the index to that collection and
	sets found to true.  If it doesn't find that resource, then it sets found to false AND sets the
	index to where it should be inserted into the list.
****************************************************************************************************/
OSErr SearchGraphicList (SInt16 resID, Boolean *found, UInt32 *index)
{	
	UInt32 low = 0;
	UInt32 high = gNumberGraphics;
	UInt32 tempIndex;
	TGraphic *theItem;
	
	OSErr theErr = noErr;
	
	*found = false;
	
	while (low < high)
		{
		tempIndex = (low+high) >> 1;
		
		theItem = (*(TGraphic ***)gTGraphicList)[tempIndex];
		
		FAIL_NIL (theItem, "\pBad TGraphic object")
		
		if (resID < theItem->GetResID())
			high = tempIndex;					//	element is below "high"
		else if (resID == theItem->GetResID())
			{
			*found = true;
			*index = tempIndex;
			return noErr;
			}
		else
			low = tempIndex+1;					//	element is above "low"
		}
	
	// use final calculations to put insert in the right place in the list
	*index = (low+high) >> 1;
	return noErr;
	
	error:
	
	if (theErr == noErr)
		theErr = paramErr;
	return theErr;


}
/****************************************************************************************************
	TGraphic::NewGraphic
	
	This routine merely uses the routines we created above to properly create and load
	the TGraphics.
****************************************************************************************************/
TGraphic
*TGraphic::NewGraphic (SInt16 resID)
{
	unsigned long		listIndex;
	Boolean 			graphicAlreadyExists;
	TGraphic			*newGraphic = NULL;
	OSErr				theErr;
	
	theErr = SearchGraphicList (resID, &graphicAlreadyExists, &listIndex);
	FAIL_OSERR(theErr,"\pCouldn't search the graphic list")
	
	if (graphicAlreadyExists)
	{
		// find the existing collection and add 1 to the ref count.
		newGraphic = (*(TGraphic ***)gTGraphicList)[listIndex];
		newGraphic->AddReference();
	}
	else
	{
		// create a new collection
		newGraphic = new TGraphic(resID);
		theErr = newGraphic->CreateGraphic ();
		FAIL_OSERR(theErr,"\pCouldn't create new TGraphic")
		theErr = InsertGraphicIntoList(newGraphic, listIndex);
		FAIL_OSERR(theErr, "\pCouldn't add new TGraphic to list")
		newGraphic->AddReference();
	}
	
	return newGraphic;
	error:
		if (newGraphic != NULL)
			delete newGraphic;
		return NULL;
}


/****************************************************************************************************
	TGraphic::AddReference
****************************************************************************************************/

void
TGraphic::AddReference (void)
   {
   	fReferenceCount++;
   }
   
   
/****************************************************************************************************
	TGraphic::DisposeReference
****************************************************************************************************/

void 
TGraphic::DisposeReference (void)
{
   	fReferenceCount--;
   	if (fReferenceCount == 0)
   	{
   		UInt32				listIndex;
		Boolean 			tableEntry;
		OSErr				theErr;

		theErr = SearchGraphicList (fResID, &tableEntry, &listIndex);
				
		FAIL_OSERR (theErr, "\pFailed to search the Graphic list")
		FAIL_FALSE (tableEntry, "\pFailed to find an existing TGraphic in list")
				
		theErr = DeleteGraphicFromList(listIndex);
		FAIL_OSERR (theErr, "\pFailed to delete graphic from the list")
				
	   	delete this;
   	}
   		
	error:
   	return;
}

   
   
/****************************************************************************************************
	TGraphic::TGraphic
	
	Just set these to some sane values
****************************************************************************************************/
TGraphic::TGraphic (SInt16 resID)
{
	fResID = resID;
	fReferenceCount = 0;
	fImage = NULL;
	fHitMask = NULL;
	
}


/****************************************************************************************************
	TGraphic::~TGraphic
	
	If we allocated a real handle, make sure we kill it.
****************************************************************************************************/
TGraphic::~TGraphic (void)
{
	DestroyGraphic();
}


/****************************************************************************************************
	TGraphic::CreateGraphic
	
	We want to try to create the graphic from a resource.  We would prefer to do this with a
	pre-encoded sprite, but we will encode a sprite on the fly if this is the only way for us
	to get it.  
	
	Basically, we'll try each of the three recognized resource formats and return as soon as one of
	them succesfully loads the TGraphic.
	
****************************************************************************************************/
OSErr TGraphic::CreateGraphic(void)
{

	
	OSErr theErr;
	
	theErr = LoadFromGraphicResource();
	
	if (theErr == noErr)
		return noErr;
	
	// Clean up any partial data, otherwise we might leak
	theErr = DestroyGraphic();
	FAIL_OSERR (theErr, "\p failed to clean up graphics")
	
	theErr = LoadFromPICTResource ();
	
	if (theErr == noErr)
		return noErr;
	
	// Clean up any partial data, otherwise we might leak.
	theErr = DestroyGraphic();
	FAIL_OSERR (theErr, "\p failed to clean up graphics")
	
	theErr = LoadFromICN8Resource ();
	
	
	if (theErr == noErr)
		return noErr;
		
	theErr = DestroyGraphic();
	FAIL_OSERR (theErr, "\p failed to clean up graphics")
	
	theErr = LoadFromCIconResource ();
	
	if (theErr == noErr)
		return noErr;
	
	(void) DestroyGraphic();
	
	error:
	return theErr;
	
}


/****************************************************************************************************
	TGraphic::DestroyGraphic
	
	All this needs to do is dispose of the image handle we created earlier.
****************************************************************************************************/

OSErr
TGraphic::DestroyGraphic (void)
{
	if (fImage != NULL)
		DisposeHandle (fImage);
	if (fHitMask != NULL)
		DisposeHandle (fHitMask);
	fImage = NULL;
	fHitMask = NULL;
	return noErr;
}



/****************************************************************************************************
	TGraphic::LoadFromGraphicResource
	
	This routine loaded a previously encoded sprite and creates all the appropriate pieces of our
	TGraphic object.
****************************************************************************************************/
OSErr TGraphic::LoadFromGraphicResource(void)
{
// first, we try to load the resource.	
// First things first, we try to load the graphic resource.
	
	Handle graphicResource = Get1Resource (kTGraphicResourceType, fResID);
	Size imageDataSize = 0;
	Size maskDataSize = 0;
	OSErr theErr = ResError();
	
	if ((theErr != noErr) || (graphicResource == NULL))
		goto error;
		
	//FAIL_OSERR (theErr, "\pFailed to load the TGraphic resource")
	//FAIL_NIL (graphicResource, "\pFailed to load the TGraphic resource")
	
	// Okay, we loaded the resource fine.  Now we want to pull all our data out of the header and 
	// into the TGraphic members.  Then we'll allocate the image and mask handles,and create the
	// image and mask handles, and copy the data over from the resource.
	
	if ((**(TGraphicResHeaderHandle)graphicResource).version != 0L)
		SIGNAL_ERROR ("\pResource format is too new to load")
		
	fBitDepth = (**(TGraphicResHeaderHandle)graphicResource).depth;
	fFlags    = (**(TGraphicResHeaderHandle)graphicResource).flags;
	fBounds   = (**(TGraphicResHeaderHandle)graphicResource).bounds;

	imageDataSize = (**(TGraphicResHeaderHandle)graphicResource).imageSize;
	maskDataSize =  (**(TGraphicResHeaderHandle)graphicResource).maskSize;
	
	fImage = NewHandle(imageDataSize);
	theErr = MemError();
	FAIL_OSERR (theErr, "\pFailed to allocate the image handle")
	FAIL_NIL   (fImage, "\pFailed to allocate the image handle")
	
	fHitMask = NewHandle (maskDataSize);
	theErr = MemError();
	FAIL_OSERR (theErr, "\pFailed to allocate the mask handle")
	FAIL_NIL   (fHitMask, "\pFailed to allocate the mask handle")

	// Copy the data into our handles and exit.
	
	BlockMoveData(((*graphicResource)+sizeof(TGraphicResHeader)), *fImage, imageDataSize);
	BlockMoveData(((*graphicResource)+sizeof(TGraphicResHeader)+imageDataSize), *fHitMask, maskDataSize);
	
	ReleaseResource (graphicResource);
	
	goto cleanup;
	
	error:
	
	if (theErr == noErr)
		theErr = paramErr;
		
	cleanup:
	if (graphicResource != NULL)
		ReleaseResource (graphicResource);
	

	return theErr;
}



/****************************************************************************************************
	TGraphic::LoadFromPICTResource
	
	This takes a previously created graphic and writes it out to the current resource fork.
****************************************************************************************************/
OSErr
TGraphic::LoadFromPICTResource (void)
{

	// first, we try to load the resource.	
	
	PicHandle thePicture = (PicHandle) Get1Resource ('PICT', fResID);
	OSErr theErr = ResError();
	GWorldPtr	graphicGWorld = NULL;
	PixMapHandle graphicPixMap = NULL;
	Rect pictRect;

	if ((theErr != noErr) || (thePicture == NULL))
		goto error;
	//FAIL_OSERR (theErr, "\pFailed to load the picture resource")
	//FAIL_NIL (thePicture, "\pFailed to load the picture resource")
	
	// grab the picture rectangle and normalize it.  It must be 3-1 aspect ratio to be a valid graphic
	
	pictRect = (**thePicture).picFrame;
	
	pictRect.right -= pictRect.left;
	pictRect.left = 0;
	pictRect.bottom -= pictRect.top;
	pictRect.top = 0;
	
	
	if (pictRect.right % 3 != 0)
		SIGNAL_ERROR ("\pImproper size pict to load as a graphic")
	
	// Create a locked GWorld with the proper depth, dimensions and color table.
	
	theErr = NewGWorld(&graphicGWorld, kPreferredDepth, &pictRect, gAppColorTable, NULL, keepLocal);
	
	FAIL_OSERR (theErr, "\pCouldn't allocate GWorld for encoding")
	
	graphicPixMap  = GetGWorldPixMap(graphicGWorld);
	FAIL_NIL (graphicPixMap, "\pCouldn't get the GWorld's PixMap")
	
	FAIL_FALSE ( LockPixels(graphicPixMap), "\pCouldn't lock GWorld PixMap")
	
	// We created the GWorld successfully -- erase it to white and draw our picture into it.
	
	{
		CGrafPtr savePort;
		GDHandle saveDevice;
		
		GetGWorld (&savePort, &saveDevice);
		SetGWorld (graphicGWorld, NULL);
	
		EraseRect (&pictRect);
		DrawPicture (thePicture, &pictRect);
	
		SetGWorld (savePort, saveDevice);
	}

	// We're done with the picture.
	ReleaseResource ((Handle) thePicture);
	thePicture = NULL;
	
	// Finally, we can pass our data along to the encoder!
	
	pictRect.right /=3;
	theErr = EncodeGraphic (graphicPixMap, &pictRect);
	
	DisposeGWorld (graphicGWorld);
	return theErr;
	error:
	
	if (thePicture != NULL)
		ReleaseResource ((Handle)thePicture);
	if (graphicGWorld != NULL)
		DisposeGWorld (graphicGWorld);
		
	if (theErr == noErr)
		theErr = paramErr;
	return theErr;

}



/****************************************************************************************************
	TGraphic::LoadFromICN8Resource
	
	This takes a previously created graphic and writes it out to the current resource fork.
****************************************************************************************************/
OSErr
TGraphic::LoadFromICN8Resource (void)
{
	// first, we try to load the resource.	
	GWorldPtr		graphicGWorld = NULL;
	PixMapHandle 	graphicPixMap = NULL;
	Handle			theIconSuite = NULL;
	RgnHandle		theMask = NULL;
	
	OSErr theErr = GetIconSuite(&theIconSuite, fResID, svAllAvailableData);

	FAIL_OSERR (theErr, "\pFailed to load the icon suite resource")
	FAIL_NIL (theIconSuite, "\pFailed to load the icon suite resource")
	
	// We'll make a GWorld the size of 3 large icons, one for each section of what would normally
	// be our "PICT".
	
	Rect pictRect = {0,0,32,96};
	Rect iconRect = {0,0,32,32};
	
	// Create a locked GWorld with the proper depth, dimensions and color table.
	
	theErr = NewGWorld(&graphicGWorld, kPreferredDepth, &pictRect, gAppColorTable, NULL, keepLocal);
	
	FAIL_OSERR (theErr, "\pCouldn't allocate GWorld for encoding")
	
	graphicPixMap  = GetGWorldPixMap(graphicGWorld);
	FAIL_NIL (graphicPixMap, "\pCouldn't get the GWorld's PixMap")
	
	FAIL_FALSE ( LockPixels(graphicPixMap), "\pCouldn't lock GWorld PixMap")
	
	// Create a region from the IconSuite's mask
	
	theMask = NewRgn();
	theErr = QDError();
	FAIL_OSERR (theErr, "\pFailed to create the mask region")
	FAIL_NIL (theMask, "\pFailed to create the mask region")
	
	theErr = IconSuiteToRgn (theMask, &iconRect, atNone, theIconSuite);
	FAIL_OSERR (theErr, "\pFailed to load the mask data into the region")
	
	// We created the GWorld successfully -- erase it to white and draw our icons into it.
	
	{
		CGrafPtr savePort;
		GDHandle saveDevice;
		
		GetGWorld (&savePort, &saveDevice);
		SetGWorld (graphicGWorld, NULL);
		
		EraseRect (&pictRect);
		
		// set the first location
		theErr = PlotIconSuite(&iconRect, atNone, ttNone, theIconSuite);
		
		OffsetRgn(theMask, 32, 0);
		FillRgn (theMask, &qd.black);
		OffsetRgn(theMask, 32, 0);
		FillRgn (theMask, &qd.black);		
	
		SetGWorld (savePort, saveDevice);
	}
	

	// We're done with the icon suite, dispose of it.
	DisposeRgn (theMask);
	theMask = NULL;
	DisposeIconSuite (theIconSuite, true);
	theIconSuite = NULL;
	
	
	// Finally, we can pass our data along to the encoder!

	theErr = EncodeGraphic (graphicPixMap, &iconRect);
	
	DisposeGWorld (graphicGWorld);
	return theErr;
	error:
	if (theMask != NULL)
		DisposeRgn (theMask);
	if (theIconSuite != NULL)
		DisposeIconSuite (theIconSuite, true);
	if (graphicGWorld != NULL)
		DisposeGWorld (graphicGWorld);
	if (theErr == noErr)
		theErr = paramErr;
	return theErr;

}



/****************************************************************************************************
	TGraphic::LoadFromICN8Resource
	
	This takes a previously created graphic and writes it out to the current resource fork.
****************************************************************************************************/
OSErr
TGraphic::LoadFromCIconResource (void)
{
	// first, we try to load the resource.	
	GWorldPtr		graphicGWorld = NULL;
	PixMapHandle 	graphicPixMap = NULL;
	CIconHandle		theCIconHdl = NULL;
	RgnHandle		theMask = NULL;
	OSErr theErr = noErr;
	
	theCIconHdl = ::GetCIcon (fResID);
	theErr = ::ResError();
	

	FAIL_OSERR (theErr, "\pFailed to load the icon suite resource")
	FAIL_NIL (theCIconHdl, "\pFailed to load the icon suite resource")
	
	// We'll make a GWorld the size of 3 large icons, one for each section of what would normally
	// be our "PICT".
	
	Rect pictRect, iconRect;
	
	iconRect = (**theCIconHdl).iconPMap.bounds;
	iconRect.right -= iconRect.left;
	iconRect.left = 0;
	iconRect.bottom -= iconRect.top;
	iconRect.top = 0;
	
	pictRect = iconRect;
	pictRect.right *= 3;
	
	// Create a locked GWorld with the proper depth, dimensions and color table.
	
	theErr = NewGWorld(&graphicGWorld, kPreferredDepth, &pictRect, gAppColorTable, NULL, keepLocal);
	
	FAIL_OSERR (theErr, "\pCouldn't allocate GWorld for encoding")
	
	graphicPixMap  = GetGWorldPixMap(graphicGWorld);
	FAIL_NIL (graphicPixMap, "\pCouldn't get the GWorld's PixMap")
	
	FAIL_FALSE ( LockPixels(graphicPixMap), "\pCouldn't lock GWorld PixMap")
	
	// Create a region from the IconSuite's mask
	
	theMask = NewRgn();
	theErr = QDError();
	FAIL_OSERR (theErr, "\pFailed to create the mask region")
	FAIL_NIL (theMask, "\pFailed to create the mask region")
	
	theErr = BitMapToRegion (theMask, &(**theCIconHdl).iconMask);
	FAIL_OSERR (theErr, "\pFailed to load the mask data into the region")
	
	// We created the GWorld successfully -- erase it to white and draw our icons into it.
	
	{
		CGrafPtr savePort;
		GDHandle saveDevice;
		
		GetGWorld (&savePort, &saveDevice);
		SetGWorld (graphicGWorld, NULL);
		
		EraseRect (&pictRect);
		
		// set the first location
		PlotCIcon (&iconRect, theCIconHdl);
		
		OffsetRgn(theMask, iconRect.right, 0);
		FillRgn (theMask, &qd.black);
		OffsetRgn(theMask, iconRect.right, 0);
		FillRgn (theMask, &qd.black);		
	
		SetGWorld (savePort, saveDevice);
	}
	

	// We're done with the icon suite, dispose of it.
	DisposeRgn (theMask);
	theMask = NULL;
	DisposeCIcon (theCIconHdl);
	theCIconHdl = NULL;
	
	
	// Finally, we can pass our data along to the encoder!

	theErr = EncodeGraphic (graphicPixMap, &iconRect);
	
	DisposeGWorld (graphicGWorld);
	return theErr;
	error:
	if (theMask != NULL)
		DisposeRgn (theMask);
	if (theCIconHdl != NULL)
		DisposeCIcon (theCIconHdl);
	if (graphicGWorld != NULL)
		DisposeGWorld (graphicGWorld);
	if (theErr == noErr)
		theErr = paramErr;
	return theErr;

}



/****************************************************************************************************
	TGraphic::EncodeGraphic
	
	This routine takes a previously prepared pixmap and encodes it into our special format.
****************************************************************************************************/
OSErr	TGraphic::EncodeGraphic (PixMapHandle theGraphic, Rect *encodeRect)
{
	OSErr theErr = noErr;
	
	unsigned short shapeHeight;				// the height of the shape
	unsigned short shapeWidth;				// the width of the shape
	
	unsigned long rowBytes;					// the rowbytes of our source pixmap
	unsigned char *baseAddr;				// the base address of the source pixmap
	
	unsigned char *dataPtr;					// points to where we are saving/encoding the data
	unsigned char *sourcePtr;				// points to the graphics data we are saving in our image
	unsigned char *maskPtr;					// points to whichever mask we are currently encoding
	
	unsigned char *sourceStartPtr;			// points to the beginning of a source row.

	unsigned long yCounter;					// a counter to scan the shape vertically
	unsigned long xCounter;					// a counter to scan the shape horizontally

	unsigned char *runTokenPtr;				// where is the token for the current run
	unsigned long runCounter;						// how long is the current run? 
	
	Size newSize = 0;
	// determine the width and height of the shape (we use these values a lot)
	// We are determining the bounds of the ACTUAL graphic, not the GWorld totals.
	shapeHeight = encodeRect->bottom - encodeRect->top;
	shapeWidth = (encodeRect->right - encodeRect->left);
	
	//Initialize the basic methods for our object.
	fBounds.left = 0;
	fBounds.top = 0;
	fBounds.right = shapeWidth;
	fBounds.bottom = shapeHeight;
	fFlags = 0;
	fBitDepth = (**theGraphic).pixelSize;
	
	if (fBitDepth != 8)
		SIGNAL_ERROR ("\pBit depths other than 8 bits are not supported yet")
		
	//create handles big enough to hold our data in the worse case scenario.
	
	fImage = NewHandle (8*shapeHeight * shapeWidth + 4*shapeHeight +4);
	theErr = MemError();
	FAIL_OSERR (theErr, "\pFailed to allocate a handle for the shape data")
	FAIL_NIL (fImage, "\pFailed to allocate a handle for the shape data")
	
	fHitMask = NewHandle (4*shapeHeight * shapeWidth + 4*shapeHeight +4);
	theErr = MemError();
	FAIL_OSERR (theErr, "\pFailed to allocate a handle for the hit mask data")
	FAIL_NIL (fHitMask, "\pFailed to allocate a handle for the hit mask data")
	
	// Get some of the standard data we'll need when encoding the pixels
	
	baseAddr = ( unsigned char * )GetPixBaseAddr( theGraphic );
	rowBytes = ( *theGraphic )->rowBytes & 0x3fff;
	
	/***********************************************************************/
	// First we'll encode the shape data.  Note that the new routine uses
	// a mask rather than watching the specific color in the object.
	// lock the handle and get the pointer
	
	HLock (fImage);
	dataPtr = ( unsigned char * )( *fImage );
	
	sourceStartPtr = baseAddr + rowBytes * encodeRect->top + encodeRect->left;
	
	// scan the shape row by row
	for( yCounter = 0; yCounter < shapeHeight; yCounter++ )
	{
		// we need to store where this line starts so we can fill in the lineStart token later.
		unsigned char *lineStartPtr;		
		lineStartPtr = dataPtr;
		dataPtr += sizeof( unsigned long );
			
		// at the beginning of each row we are not in any run
		unsigned char drawRunFlag = false;				// are we in a draw pixels run?
		unsigned char skipRunFlag = false;				// are we in a skip pixels run?
		
		// move to the start of the row
		sourcePtr = sourceStartPtr;
		maskPtr = sourceStartPtr + shapeWidth;
			
		// scan each row of the shape
		for( xCounter = 0; xCounter < shapeWidth; xCounter++ )
		{
		// check the mask to see if this is a clear pixel or not.
					// is this pixel clear?
			if ( *maskPtr == kClearColorIndex )
			{
				// are we in a draw run?
				if ( drawRunFlag )
				{
					// end the draw run
					drawRunFlag = false;
									
					// create the draw token
					*( ( unsigned long * )runTokenPtr ) = ( kDrawPixelsToken << 24 ) + runCounter;
									
					// clear and pad to a mulitple of four
					*( ( unsigned long * )dataPtr ) = 0L;
					dataPtr = (unsigned char *) ALIGN_TO_NEXT_LONG(dataPtr);
				}
							
				// are we in a skip run
				if ( skipRunFlag )
				{
				// continue it
					runCounter++;
				}
				else
				{
					// start one
					skipRunFlag = true;
					runCounter = 1;
				}
			}
			else
			{
			// are we in a skip run
				if ( skipRunFlag )
				{
					// end the skip run
					skipRunFlag = false;
					// create the skip token
					*( ( unsigned long * )dataPtr ) = ( kSkipPixelsToken << 24 ) + runCounter;
					dataPtr += sizeof( unsigned long );
				}
							
				// are we in a draw run
				if ( drawRunFlag )
				{
					// continue it
					runCounter++;
									
					// copy the pixel
					*dataPtr = *sourcePtr;
					dataPtr++;
				}
				else
				{
					// start one
					drawRunFlag = true;
					runCounter = 1;
									
					// save the location of the token (so we can fill it in later)
					runTokenPtr = dataPtr;
					dataPtr += sizeof( unsigned long );
									
					// copy the pixel
					*dataPtr = *sourcePtr;
					dataPtr++;
				}
			}
					
			// move to the next byte
			sourcePtr++;
			maskPtr++;
		}
		
		// if we're finishing up a draw run, we need to write out the final data.	
			// are we in a draw run
		if( drawRunFlag )
		{
			// create the draw token
			*( ( unsigned long * )runTokenPtr ) = ( kDrawPixelsToken << 24 ) + runCounter;
									
			// clear and pad to a mulitple of four
			*( ( unsigned long * )dataPtr ) = 0L;
			dataPtr = (unsigned char *) ALIGN_TO_NEXT_LONG(dataPtr);
		}
			
		// create the line start token
		*( ( unsigned long * )lineStartPtr ) = ( kLineStartToken << 24 ) + ( dataPtr - ( lineStartPtr + 4 ) );
			
		// move the row start to the next row
		sourceStartPtr += rowBytes;
	}
	
	// create the end of shape token
	*( ( unsigned long * )dataPtr ) = kEndShapeToken << 24;
	dataPtr += sizeof( unsigned long );
	
	newSize = dataPtr - ( unsigned char * )( *fImage );
	
	// Unlock the handle
	HUnlock( fImage );
	
	// Resize the handle to match the real size of the shape
	SetHandleSize( fImage, newSize );
	theErr = MemError();
	FAIL_OSERR (theErr, "\p Failed to resize the handle")
	

	/***********************************************************************/
	// Now we'll encode the mask.  This code is almost identical, except that we don't keep track of 
	// the source data at all since we don't need to copy it into the hit mask shape.


	HLock (fHitMask);
	dataPtr = ( unsigned char * )( *fHitMask );
	
	sourceStartPtr = baseAddr + rowBytes * encodeRect->top + encodeRect->left + shapeWidth+shapeWidth;
	
	// scan the shape row by row
	for( yCounter = 0; yCounter < shapeHeight; yCounter++ )
	{
		// we need to store where this line starts so we can fill in the lineStart token later.
		unsigned char *lineStartPtr;		
		lineStartPtr = dataPtr;
		dataPtr += sizeof( unsigned long );
			
		// at the beginning of each row we are not in any run
		unsigned char drawRunFlag = false;				// are we in a draw pixels run?
		unsigned char skipRunFlag = false;				// are we in a skip pixels run?
		
		maskPtr = sourceStartPtr;
			
		// scan each row of the shape
		for( xCounter = 0; xCounter < shapeWidth; xCounter++ )
		{
		// check the mask to see if this is a clear pixel or not.
					// is this pixel clear?
			if ( *maskPtr == kClearColorIndex )
			{
				// are we in a draw run?
				if ( drawRunFlag )
				{
					// end the draw run
					drawRunFlag = false;
									
					// create the draw token
					*( ( unsigned long * )dataPtr ) = ( kDrawPixelsToken << 24 ) + runCounter;
					dataPtr += sizeof( unsigned long );
				}
							
				// are we in a skip run
				if ( skipRunFlag )
				{
				// continue it
					runCounter++;
				}
				else
				{
					// start one
					skipRunFlag = true;
					runCounter = 1;
				}
			}
			else
			{
			// are we in a skip run
				if ( skipRunFlag )
				{
					// end the skip run
					skipRunFlag = false;
					// create the skip token
					*( ( unsigned long * )dataPtr ) = ( kSkipPixelsToken << 24 ) + runCounter;
					dataPtr += sizeof( unsigned long );
				}
							
				// are we in a draw run
				if ( drawRunFlag )
				{
					// continue it
					runCounter++;
				}
				else
				{
					// start one
					drawRunFlag = true;
					runCounter = 1;
				}
			}
					
			// move to the next byte
					maskPtr++;
		}
		
		// if we're finishing up a draw run, we need to write out the token

		if( drawRunFlag )
		{
			*( ( unsigned long * )dataPtr ) = ( kDrawPixelsToken << 24 ) + runCounter;
			dataPtr += sizeof( unsigned long );
		}
			
		// create the line start token
		*( ( unsigned long * )lineStartPtr ) = ( kLineStartToken << 24 ) + ( dataPtr - ( lineStartPtr + 4 ) );
			
		// move the row start to the next row
		sourceStartPtr += rowBytes;
	}
	
	// create the end of shape token
	*( ( unsigned long * )dataPtr ) = kEndShapeToken << 24;
	dataPtr += sizeof( unsigned long );
	
	newSize = dataPtr - ( unsigned char * )( *fHitMask );
	
	// Unlock the handle
	HUnlock( fHitMask );
	
	// Resize the handle to match the real size of the shape
	SetHandleSize( fHitMask, newSize );

	theErr = MemError();
	FAIL_OSERR (theErr, "\p Failed to resize the handle")

	return noErr;

	error:
	
	if (theErr == noErr)
		theErr = paramErr;
	return theErr;
}
	
	

/****************************************************************************************************
	TGraphic::WriteToGraphicResource
	
	This takes a previously created graphic and writes it out to the current resource fork.
****************************************************************************************************/
OSErr
TGraphic::WriteToGraphicResource (void)
{
	Handle	graphicResource = NULL;
	OSErr	theErr;
	Size imageSize;
	Size maskSize;
	
	imageSize = GetHandleSize(fImage);
	theErr = MemError();
	FAIL_OSERR (theErr, "\pCouldn't obtain image handle size")
	
	maskSize = GetHandleSize (fHitMask);
	theErr = MemError();
	FAIL_OSERR (theErr, "\pCouldn't obtain mask handle size")
	
	graphicResource = NewHandle (sizeof(TGraphicResHeader) + imageSize + maskSize);
	theErr = MemError();
	FAIL_OSERR (theErr, "\pCouldn't allocate handle to hold new resource")
	FAIL_NIL (graphicResource, "\pCouldn't allocate handle to hold new resource")
	
	// Now to fill in the header and block move all of the data over into the new resource.
	
	(**(TGraphicResHeaderHandle)graphicResource).version   = 0L;
	(**(TGraphicResHeaderHandle)graphicResource).depth     = fBitDepth;
	(**(TGraphicResHeaderHandle)graphicResource).flags     = fFlags;
	(**(TGraphicResHeaderHandle)graphicResource).bounds    = fBounds;
	(**(TGraphicResHeaderHandle)graphicResource).imageSize = imageSize;
	(**(TGraphicResHeaderHandle)graphicResource).maskSize  = maskSize;
	
	BlockMoveData(*fImage, ((*graphicResource)+sizeof(TGraphicResHeader)), imageSize);
	BlockMoveData(*fHitMask, ((*graphicResource)+sizeof(TGraphicResHeader)+imageSize), maskSize);
	
	// **TO DO:  Check to see if there is already a resource with that ID!
	
	// Add the resource to the file.
	AddResource( graphicResource, kTGraphicResourceType, fResID, "\p" );
	theErr = ResError();
	FAIL_OSERR (theErr, "\pFailed to add the TGraphic resource to the file")
	
	WriteResource( graphicResource );
	theErr = ResError();
	
	ReleaseResource( graphicResource );
	
	return theErr;
	
	error:
	if (graphicResource != NULL)
		DisposeHandle (graphicResource);
	if (theErr == noErr)
		theErr = paramErr;
	return theErr;
}



/****************************************************************************************************
	TGraphic::WriteToPICTResource
	
	This takes a previously created graphic and writes it out to the current resource fork as a PICT
****************************************************************************************************/
OSErr
TGraphic::WriteToPICTResource (void)
{
	// First, create a GWorld to hold the picture information
	
	Rect pictRect = fBounds;
	OSErr theErr = NULL;
	GWorldPtr graphicGWorld = NULL;
	PixMapHandle graphicPixMap = NULL;
	PicHandle	thePicture = NULL;
	
	// need room for 3 pictures
	pictRect.right += (pictRect.right+pictRect.right);
	
	// Create a locked GWorld with the proper depth, dimensions and color table.
	
	theErr = NewGWorld(&graphicGWorld, kPreferredDepth, &pictRect, gAppColorTable, NULL, keepLocal);
	
	FAIL_OSERR (theErr, "\pCouldn't allocate GWorld for encoding")
	
	graphicPixMap  = GetGWorldPixMap(graphicGWorld);
	FAIL_NIL (graphicPixMap, "\pCouldn't get the GWorld's PixMap")
	
	FAIL_FALSE ( LockPixels(graphicPixMap), "\pCouldn't lock GWorld PixMap")
	
	// We created the GWorld successfully -- erase it to white and draw our graphics into it.
	
	// We'll save off the previous data so that we can temporarily set up our own draw context.
	// This code takes a lot of advantage of our current data formats, and would need to be modified
	// heavily if we do more work in PrepareDrawing and EndDrawing.
	
	{
		CGrafPtr savePort;
		GDHandle saveDevice;
		
		GetGWorld (&savePort, &saveDevice);
		SetGWorld (graphicGWorld, NULL);
		
		ClipRect (&pictRect);
	
		Rect 				saveRect = gClipRect;
		PixMapHandle		saveDestPixMap = gDestPixMap;
		PixMapHandle    	saveBackPixMap = gBackPixMap;
		unsigned char		*saveDestBaseAddr = gDestBaseAddr;
		unsigned char    	*saveBackBaseAddr = gBackBaseAddr;
		unsigned long		saveRowBytes = gRowBytes;

		SetDestinationBuffer (graphicPixMap, NULL);

		EraseRect (&pictRect);
		
		// finally we get to draw.
		
		SInt32 top, left;
		top = fBounds.top;
		left = fBounds.left;
		
		GraphicUnclipped (top, left);
		left += fBounds.right;
		DrawMaskUnclipped (top, left);
		left += fBounds.right;
		HitMaskUnclipped (top, left);
		
		// restore all our saved junk
		
		gClipRect = saveRect;
		gDestPixMap = saveDestPixMap;
		gBackPixMap = saveBackPixMap;
		gDestBaseAddr = saveDestBaseAddr;
		gBackBaseAddr = saveBackBaseAddr;
		gRowBytes = saveRowBytes;
		
		//Okay, we can create the PICT now.
		
		OpenCPicParams theParams;
		
		theParams.srcRect = pictRect;
		theParams.hRes = 0x00480000;
		theParams.vRes = 0x00480000;
		theParams.version = -2;
		theParams.reserved1 = 0;
		theParams.reserved2 = 0;
		
		thePicture = OpenCPicture (&theParams);
		
		FAIL_NIL (thePicture, "\p Couldn't open the new picture")
		CopyBits ((BitMap *) *graphicPixMap, (BitMap *) *graphicPixMap, &pictRect, &pictRect, srcCopy, NULL);
		ClosePicture();
		theErr = QDError();
		
		FAIL_OSERR (theErr, "\pFailed to create new picture")
		
		SetGWorld (savePort, saveDevice);
	}
	
	// close up the GWorld, we're done with it
	DisposeGWorld (graphicGWorld);
	graphicGWorld = NULL;
	
	// And add our new pict to the resource fork.
	
	AddResource( (Handle) thePicture, 'PICT', fResID, "\p" );
	theErr = ResError();
	FAIL_OSERR (theErr, "\pFailed to add the PICT resource to the file")
	
	WriteResource( (Handle) thePicture );
	theErr = ResError();
	
	ReleaseResource( (Handle) thePicture );
	
	return theErr;

	// clean up and exit

	return theErr;
	error:
	
	if (thePicture != NULL)
		KillPicture (thePicture);
	if (graphicGWorld != NULL)
		DisposeGWorld (graphicGWorld);
	if (theErr == noErr)
		theErr = paramErr;
	return theErr;



}



/****************************************************************************************************
	TGraphic::LockGraphic
	
	All this needs to do is dispose of the image handle we created earlier.
****************************************************************************************************/
OSErr
TGraphic::LockGraphic (void)
{
	OSErr theErr;
	MoveHHi (fImage);
	theErr = MemError();
	FAIL_OSERR (theErr, "\pFailed to move the draw data high")
	HLock (fImage);
	theErr = MemError();
	FAIL_OSERR (theErr, "\pFailed to lock the draw data")

	MoveHHi (fHitMask);
	theErr = MemError();
	FAIL_OSERR (theErr, "\pFailed to move the mask data high")
	HLock (fHitMask);
	theErr = MemError();
	FAIL_OSERR (theErr, "\pFailed to lock the mask data")	
	return noErr;
	
	error:
	return theErr;
}


/****************************************************************************************************
	TGraphic::LockGraphic
	
	All this needs to do is dispose of the image handle we created earlier.
****************************************************************************************************/

OSErr
TGraphic::UnlockGraphic (void)
{
	OSErr theErr;
	HUnlock (fImage);
	theErr = MemError();
	FAIL_OSERR (theErr, "\pFailed to unlock the draw data")
	HUnlock (fHitMask);
	theErr = MemError();
	FAIL_OSERR (theErr, "\pFailed to unlock the draw data")	
	return noErr;
	
	error:
	
	return theErr;
}






/****************************************************************************************************
	TGraphic::CopyImage
	
	This routine compares are drawing location to the clipping region, and dispatches to the correct
	drawing utility routines.

****************************************************************************************************/
void
TGraphic::CopyImage (SInt32 top, SInt32 left, Boolean useBackground)
{
#if qDebugging
	if (gDestPixMap == NULL)
		SIGNAL_ERROR ("\pAttempting to draw to a NULL destination pixmap")
		
	if (useBackground && (gBackPixMap == NULL))
		SIGNAL_ERROR ("\pAttepting to draw from an NULL background pixmap.")
		
	if (EmptyRect (&gClipRect))
		SIGNAL_ERROR ("\pEmpty Clipping Region")
#endif
	// We'll do all our calculations in variables rather than use a structure.
	// hopefully this should speed things up
	SInt32 destTop, destBottom, destLeft, destRight;
	
	destTop    = fBounds.top + top;
	destBottom = fBounds.bottom + top;
	destLeft   = fBounds.left + left;
	destRight  = fBounds.right + left;
	
	// determine if the spite needs to be drawn at all
	if	(destTop >= gClipRect.bottom || destBottom <= gClipRect.top ||
		 destLeft >= gClipRect.right || destRight <= gClipRect.left )
		// no need to draw, goodbye
		return;
	
	// determine if the sprite will be clipped
	if 	(destTop < gClipRect.top || destBottom > gClipRect.bottom ||
		 destLeft < gClipRect.left || destRight > gClipRect.right)
		{
		// calculate the clipped bounding box in object local coordinates
		UInt32 clipTop, clipBottom, clipLeft, clipRight;
				
		clipTop    = destTop < gClipRect.top       ? gClipRect.top - destTop  : 0;
		clipBottom = destBottom > gClipRect.bottom ? gClipRect.bottom-destTop : destBottom-destTop;
		clipLeft   = destLeft < gClipRect.left     ? gClipRect.left-destLeft  : 0;
		clipRight  = destRight > gClipRect.right   ? gClipRect.right-destLeft : destRight-destLeft;

		if (useBackground)
			BackgroundClipped (destTop, destLeft, clipLeft, clipRight, clipTop, clipBottom);
		else
			GraphicClipped(destTop, destLeft, clipLeft, clipRight, clipTop, clipBottom );
		}
	else
		{
		if (useBackground)
			BackgroundUnclipped (destTop, destLeft);
		else
			GraphicUnclipped(destTop, destLeft );
		}

error:
	return;
}


/****************************************************************************************************
	TGraphic::HitTest
	
	This hit testing code walks the Graphic's data and returns true if the point intersects a
	drawing command.

	Basic algorithm:
	
	First, we check to make sure that the point is even within the graphic's rectangle.  If it isn't,
	then we can automatically return false.
	
	After that, we look for drawing commands. Note that on the right side, we test greater than
	or equal, and on the left we only test less than.  This is because the point that corresponds to a coord
	goes down and to the left.  Think about it and you'll understand. :-)
****************************************************************************************************/
Boolean
TGraphic::HitTest (SInt32 v, SInt32 h)
{	
	if( v >= fBounds.bottom || v < fBounds.top ||
	    h >= fBounds.right || h < fBounds.left )
	{
		return false;
	}
	
	// Our standard things to read the sprite
	unsigned char *dataPtr;
	unsigned long tokenOp;
	unsigned long tokenData;
	
	// We use these to count where we are in the image, to move to our point.
		
	unsigned long yCount = 0;
	unsigned long xCount = 0; 
	
	// Move to the start of our hit testing data
	dataPtr = ( unsigned char * )(*fHitMask);

	// Do a quick walk to the line which holds the point.
	while (v > 0)
	{
		v--;
		tokenData = ( *( ( unsigned long * )dataPtr ) ) & 0x00ffffff;
			
		dataPtr += sizeof( unsigned long );
		dataPtr += tokenData;
	}
	
	// Skip the newLine token
	dataPtr += sizeof( unsigned long );
		
	// Start reading draw commands.  Any end or newLine tokens mean we're done.
	// We'll exit out of the While loop via a return.
	do 
	{
		// get a token
		tokenOp = ( *( ( unsigned long * )dataPtr ) ) >> 24;
		tokenData = ( *( ( unsigned long * )dataPtr ) ) & 0x00ffffff;
		dataPtr += sizeof( unsigned long );
			
		// depending on the token, we take an action
		
		// Note that line starts will be very common to begin with, so we'll move them up to the top
		switch( tokenOp )
		{
			case kHitTestPixelsToken:

	// We can only generate a hit in a draw command.  If our point is between
	// xCount and xCount+tokenData, then it is on a drawn line, so return true.
				if ((xCount <= h) && (xCount+tokenData>h))
					return true;
				else if (xCount > h)
					return false;
				xCount += tokenData;
				
				break;
					
			case kSkipPixelsToken:
					
				xCount += tokenData;
						
				if (xCount > h)
					return false;
							
				break;
					
			case kLineStartToken:
			case kEndShapeToken:
				return false;
#if qDebugging		
			default:
				SIGNAL_ERROR ("\pInvalid TGraphic drawing operation")
				break;
#endif
		}
	} while ( true );

error:
return false;
}



/****************************************************************************************************
	TGraphic::Intersect
	
	This compares two TGraphic objects and two points in the SAME coordinate system and returns
	true if the two objects intersect.  Only DRAWING commands intersect.
****************************************************************************************************/


Boolean TGraphic::Intersect (TGraphic *object1, TGraphic *object2, SInt32 v1, SInt32 h1, SInt32 v2, SInt32 h2)
{
	SInt32 		differenceX, differenceY;
	
	SInt32		object1right, object1bottom;
	SInt32		object2top, object2bottom, object2left, object2right;
	
	UInt32		xIntersectLeft, xIntersectRight, yHeight;
	
	
	// First, we sort the two objects in order of the x coordinate.
	
	if (h2 < h1)
	{
		SInt32 tempNum;
		TGraphic *tempGraphic;
		
		tempGraphic = object1; object1 = object2; object2 = tempGraphic;

		tempNum = h2; h2 = h1; h1 = tempNum;
		tempNum = v2; v2 = v1; v1 = tempNum;
	}

	// Next we calculate the bounding boxes of the two graphics and check to see if they overlap.
	// if they don't we can drop out and exit.
	
	differenceX = h2-h1;
	differenceY = v2-v1;
	// create two rectangles based on the object locations.
	
	object1right = (object1->fBounds).right;
	object1bottom = (object1->fBounds).bottom;
	
	object2left = differenceX;
	object2right = object2->fBounds.right + differenceX;
	object2top = differenceY;
	object2bottom = object2->fBounds.bottom + differenceY;

	if (object2bottom <= 0 || object2top >= object1bottom ||
		object2left >= object1right)
		return false;

	// Next we want to do some calculations that we'll use to parse the list.  Basically, we are
	// determining the coordinates we're going to use for to search for an intersection.
	
	xIntersectLeft = object2left;
	xIntersectRight = object1right > object2right  ? object2right  :  object1right;
	
	{
		UInt32 yTop, yBottom;
	
		yTop    = object2top    > 0             ? object2top     :     0;
		yBottom = object1bottom > object2bottom ? object2bottom  :     object1bottom;

		yHeight = yBottom-yTop; // the number of row's we'll need to test.
	}
			
// These will hold whichever command we are looking at.
	UInt32 tokenOp;
	UInt32 tokenData;
	
	// These will hold our current pointers to commands for both objects.
	unsigned char *dataPtr1, *dataPtr2;
	
	// We'll precalculate the pointers for the next line, to make it easier to change
	// both objects at the same time.
	
	unsigned char *nextDataPtr1, *nextDataPtr2;
	
	// Whenever we hit an end of line, we need to drop out and make sure both data pointers are incremented.
		
	unsigned char lineDone = false;
	
	// We occasionally need to drop out of just the inner loop -- we'll set this to true.
	
	unsigned char innerObjectDone = false;
	
	// These count where we're at in each of the two object's scanlines.
	
	UInt32 xCount1 = 0, xCount2 = 0;



	// move to the start of each shape's data.
	dataPtr1 = ( unsigned char * )(*(object1->fHitMask));
	dataPtr2 = ( unsigned char * )(*(object2->fHitMask));

	// One of the two objects will probably need to be advanced in the vertical coordinate to put us
	// in the correct position.  We'll count out a number of skips equal to the previous y difference
	// we calculated.
	// now we need to skip lines until we get to the correct starting location for each shape.
	// With the current encoder, all objects start with startline commands.  So we'll cound out a number
	// of skips equal to the intersection rect's top Y coordinate.
	if (differenceY > 0)
		do
		{
			tokenData = ( *( ( unsigned long * )dataPtr1 ) ) & 0x00ffffff;
			
			dataPtr1 += sizeof( unsigned long );
			dataPtr1 += tokenData;
		}
		while (--differenceY > 0);
	else
		while (differenceY++ < 0)
		{
			tokenData = ( *( ( unsigned long * )dataPtr2 ) ) & 0x00ffffff;
			
			dataPtr2 += sizeof( unsigned long );
			dataPtr2 += tokenData;
		}
	// Now we're ready to begin.  For the most part, we loop inside object1 looking for a draw command
	// inside the intersected area.  If we find one, then we run a second loop inside this, looking
	// for a draw commmand in object2 that intersects the one from object1.  If we find one, we return
	// true.  If we hit the end of the shape, or the intersected area, we return false.
	
	// If we hit the end of a particular line, then we drop out of both loops and advance the pointers
	// appropriately, and run the next line.
	
	do 
	{
	// First, we retrieve the start of line command for each of the objects -- from this we can
	// calculate the pointers for the next scanline, which we use when we drop out of the search
	// loops.  
		tokenOp = ( *( ( unsigned long * )dataPtr1 ) ) >> 24;
		tokenData = ( *( ( unsigned long * )dataPtr1 ) ) & 0x00ffffff;
			
		dataPtr1 += sizeof( unsigned long );
		nextDataPtr1 = dataPtr1+ tokenData;
		
		tokenOp = ( *( ( unsigned long * )dataPtr2 ) ) >> 24;
		tokenData = ( *( ( unsigned long * )dataPtr2 ) ) & 0x00ffffff;
			
		dataPtr2 += sizeof( unsigned long );
		nextDataPtr2 = dataPtr2+ tokenData;
		
		// reset our booleans and our horizontal positions.
		lineDone = false;
		xCount1 = 0;
		xCount2 = xIntersectLeft;
		

		
		while (!lineDone)
		{
		// Loop in object1 looking for a draw command inside the intersected area.  If we hit the end
		// of our intersected area or a new line token, we'll drop out of the loop.
		// Get the next operation
			tokenOp = ( *( ( unsigned long * )dataPtr1 ) ) >> 24;
			tokenData = ( *( ( unsigned long * )dataPtr1 ) ) & 0x00ffffff;
			
			dataPtr1 += sizeof( unsigned long );
		
			switch (tokenOp)
			{
				case kHitTestPixelsToken:
					// kDrawPixels is the ugly token to parse, because if the drawing command
					// is inside the intersected area, we have to run ANOTHER loop for object 2.
					{
					unsigned long x1RightDraw = xCount1+tokenData;
					unsigned long x1LeftDraw = xCount1;
					// skip to next command
					
					// if we're still to the left of our intersection, skip to next set
					if (x1RightDraw <= xIntersectLeft)
						break;
						
					// pin left side to intersection
					if (x1LeftDraw < xIntersectLeft)
						x1LeftDraw = xIntersectLeft;
						
					// pin to right side to intersection
					if (x1RightDraw > xIntersectRight)
						x1RightDraw = xIntersectRight;
					
					// Now we loop inside of object 2 until we find an intersecting draw command
					// or skip past object1's draw area.
					innerObjectDone = false;
					
					while (!(lineDone || innerObjectDone)) 
					{
						tokenOp = ( *( ( unsigned long * )dataPtr2 ) ) >> 24;
						tokenData = ( *( ( unsigned long * )dataPtr2 ) ) & 0x00ffffff;
						
						switch (tokenOp)
						{
							case kHitTestPixelsToken:
								// Now we can finally check for an intersection!!
									
								if (!(((xCount2 <= x1LeftDraw) && (xCount2+tokenData <= x1LeftDraw)) ||
									  ((xCount2 >= x1RightDraw) && (xCount2+tokenData > x1RightDraw))))
									 return true;
									
								// if we are to the right of object1's drawing command, then we drop out of
								// the object2 loop.  Note that in this case we do not advance to the next
								// draw command in object2, as we might need to check the next drawing command
								// in object1 against it.
									
								if (xCount2+tokenData >= x1RightDraw)
								{
									innerObjectDone = true;
								}
								else
								{
									xCount2 += tokenData;
									dataPtr2 += sizeof( unsigned long );
								}
								break;
								
							case kSkipPixelsToken:
								xCount2 += tokenData;
								dataPtr2 += sizeof( unsigned long );
								if (xCount2 > xIntersectRight)
									innerObjectDone = true;
								break;
								
							case kLineStartToken:
								lineDone = true;
								break;
							case kEndShapeToken:
								return false;
								break;
#if qDebugging
							default:
								Debugger();
								break;
#endif
						} // end inner switch
					} // end inner while loop
					
					// and make sure we exit if were at the end of the line in object1.
					if (xCount1 >= xIntersectRight)
						lineDone = true;
					}
					break;
					
				case kSkipPixelsToken:
					xCount1 += tokenData;
					if (xCount1 >= xIntersectRight)
						lineDone = true;
					break;
				case kLineStartToken:
					lineDone = true;
					break;
				case kEndShapeToken:
					return false;
					break;
#if qDebugging
				default:
					Debugger();
					break;
#endif
			}
		}			
		// skip to the next line for both and increment the count.
		dataPtr1 = nextDataPtr1;
		dataPtr2 = nextDataPtr2;
	} while (--yHeight > 0);
	
	// If we got this far, we've dropped out of the bottom of the intersect area. 
	return false;
}



/****************************************************************************************************
	TGraphic::GraphicClipped
****************************************************************************************************/
	
void
TGraphic::GraphicClipped (SInt32 top, SInt32 left, UInt32 clipLeft, UInt32 clipRight, UInt32 clipTop, UInt32 clipBottom)

// This routine is almost identical to the one from Tips of the Mac Game Programming Gurus.
{
	unsigned char *rowStart;				// the pointer to the start of this row
	unsigned char *srcPtr;					// the current position in the sprite data
	unsigned char *destPtr;					// the current position in the destination pixmap
	unsigned long miscCounter;				// a counter for various purposes
	unsigned long extraCounter;				// a counter for right clippling purposes ( how much extra was there? )
	unsigned long tokenOp;					// the op code from the token
	unsigned long tokenData;				// the data from the token
	unsigned long yCount;					// how many lines do we have left to scan.
	unsigned long xCount;					// where are we in this line?
	
	// Determine the actual clipping area we're going to need to draw, we do these as unsigned longs because
	// that's what we'll use when drawing.
		
	// set up the initial count of rows we'll want to search
	yCount = clipBottom-clipTop;

	// determine characteristics about the pixmap
	rowStart = gDestBaseAddr + (top + clipTop) * gRowBytes + left;

	// move to the start of the image data.  We'll do a quick skip of any initial pixel rows that we don't
	// need to worry about.
	srcPtr = ( unsigned char * )( *fImage);

	while (clipTop > 0)
	{
		clipTop--;
		tokenData = ( *( ( unsigned long * )srcPtr ) ) & 0x00ffffff;
		srcPtr += sizeof( unsigned long ) + tokenData;
	}
	
	// start the loop
	do
	{
		// get a token
		tokenOp = ( *( ( unsigned long * )srcPtr ) ) >> 24;
		tokenData = ( *( ( unsigned long * )srcPtr ) ) & 0x00ffffff;
		srcPtr += sizeof( unsigned long );
			
		// depending on the token
		switch( tokenOp )
		{
			case kDrawPixelsToken:
				miscCounter = tokenData;
				extraCounter = 0;
						
				// if we need to, clip to the left
				if( xCount < clipLeft )
				{
					// if this run does not appear at all, don't draw it
					if ( miscCounter < clipLeft - xCount )
					{
						destPtr += miscCounter;
						xCount += miscCounter;
						miscCounter = ALIGN_TO_NEXT_LONG(miscCounter);					
						srcPtr += miscCounter;

						break;
					}
					else
					{
						// if it does, skip to where we can draw
						miscCounter -= clipLeft - xCount;
						destPtr += clipLeft - xCount;
						srcPtr += clipLeft - xCount;
						xCount = clipLeft;
					}
				}
						
				// if we need to, clip to the right
				if ( xCount + miscCounter > clipRight )
				{
					// if this run does not appear at all, skip it
					if ( xCount > clipRight )
					{
						// once xCount is greater than xRight, no drawing commands will ever
						// be issued on that line, so I changed the "gurus" code, which was actually
						// wasting a couple of instructions setting variables that aren't needed.
						//destPtr += miscCounter;
						//xCount += miscCounter;
						miscCounter = ALIGN_TO_NEXT_LONG(miscCounter);					
						srcPtr += miscCounter;
						break;
					}
					else
					{
						// if it does, setup to draw what we can
						extraCounter = (xCount + miscCounter) - clipRight;
						miscCounter -= extraCounter;
					}
				}
				
				// adjust xCount for the run
				xCount += miscCounter;
						
				// This blit loop is optimized to move 32 bytes or less.  On 604 machines, using an 8 byte move
				// is faster in those rare cases where you can get them to align, and way slower when they don't.
				// because of the rarity of these cases, this blit loop doesn't do anything fancy with doubles.
				
				
				{
				register unsigned long sixteenblits, blitloop;
				sixteenblits = miscCounter >> 4;
				for ( blitloop = 0; blitloop < sixteenblits; blitloop++)
				{
						register unsigned long temp1, temp2, temp3, temp4;
						temp1 = ((unsigned long *) srcPtr)[0];
						temp2 = ((unsigned long *) srcPtr)[1];
						temp3 = ((unsigned long *) srcPtr)[2];
						temp4 = ((unsigned long *) srcPtr)[3];
						((unsigned long *) destPtr)[0] = temp1;
						((unsigned long *) destPtr)[1] = temp2;
						((unsigned long *) destPtr)[2] = temp3;
						((unsigned long *) destPtr)[3] = temp4;
						srcPtr += 16;
						destPtr += 16;
				}
				// move any remaining data, up to 15 bytes total
				if (miscCounter & 0x8)
				{
					register unsigned long temp1, temp2;
					temp1 = ((unsigned long *) srcPtr)[0];
					temp2 = ((unsigned long *) srcPtr)[1];
					((unsigned long *) destPtr)[0] = temp1;
					((unsigned long *) destPtr)[1] = temp2;
					srcPtr += 8;
					destPtr += 8;
				}
				if (miscCounter & 0x4)
				{
					register unsigned long temp1;
					temp1 = *((unsigned long *) srcPtr);
					srcPtr +=4;
					*((unsigned long *) destPtr)  = temp1;
					destPtr +=4;
				}
				if (miscCounter & 0x2)
				{
					register unsigned short temp1;
					temp1 = *((unsigned short *) srcPtr);
					srcPtr +=2;
					*((unsigned short *) destPtr)  = temp1;
					destPtr +=2;
				}
				if (miscCounter & 0x1)
					*destPtr++ = *srcPtr++;
		
				}
						
				// adjust for right clipping
				destPtr += extraCounter;
				srcPtr += extraCounter;
				xCount += extraCounter;
						
				// adjust for the padding
				srcPtr = (unsigned char *) ALIGN_TO_NEXT_LONG(srcPtr);	
				break;
						
			case kSkipPixelsToken:
				destPtr += tokenData;
				xCount += tokenData;
				break;
						
			case kLineStartToken:
				// check to see if we're done drawing
				if (yCount == 0)
					return;
				else
				{
					// set all the counters up
					yCount--;
					xCount = 0;
					// set up the destination pointer
					destPtr = rowStart;
					rowStart += gRowBytes;
				}
				break;
						
			case kEndShapeToken:
				// exit.
				return;
				break;
#if qDebugging						
			default:
				// we should never get here
				Debugger();
				break;
#endif
		}
	} while (true);
}


/****************************************************************************************************
	TGraphic::GraphicUnclipped
****************************************************************************************************/

void 
TGraphic::GraphicUnclipped (SInt32 top, SInt32 left)
{
// This routine is almost identical to the one from Tips of the Mac Game Programming Gurus.

	unsigned char *rowStart;				// the pointer to the start of this row
	unsigned char *srcPtr;						// the current position in the sprite data
	unsigned char *destPtr;					// the current position in the destination pixmap
	unsigned long miscCounter;			// a counter for various purposes
	unsigned long tokenOp;							// the op code from the token
	unsigned long tokenData;					// the data from the token
	
	// determine characteristics about the pixmap
	rowStart = gDestBaseAddr + top * gRowBytes + left;
	
	// move to the start of the image data.
	srcPtr = ( unsigned char * )( *fImage );
	
	// Start looping
	do
	{
		// get a token
		tokenOp = ( *( ( unsigned long * )srcPtr ) ) >> 24;
		tokenData = ( *( ( unsigned long * )srcPtr ) ) & 0x00ffffff;
		srcPtr += sizeof( unsigned long );
			
		// depending on the token
		switch( tokenOp )
		{
			case kDrawPixelsToken:
				miscCounter = tokenData;
				
				// This blit loop is optimized to move 32 bytes or less.  On 604 machines, using an 8 byte move
				// is faster in those rare cases where you can get them to align, and way slower when they don't.
				// because of the rarity of these cases, this blit loop doesn't do anything fancy with doubles.
				
				
				{
				register unsigned long sixteenblits, blitloop;
				sixteenblits = miscCounter >> 4;
				for ( blitloop = 0; blitloop < sixteenblits; blitloop++)
				{
						register unsigned long temp1, temp2, temp3, temp4;
						temp1 = ((unsigned long *) srcPtr)[0];
						temp2 = ((unsigned long *) srcPtr)[1];
						temp3 = ((unsigned long *) srcPtr)[2];
						temp4 = ((unsigned long *) srcPtr)[3];
						((unsigned long *) destPtr)[0] = temp1;
						((unsigned long *) destPtr)[1] = temp2;
						((unsigned long *) destPtr)[2] = temp3;
						((unsigned long *) destPtr)[3] = temp4;
						srcPtr += 16;
						destPtr += 16;
				}
				// move any remaining data, up to 15 bytes total
				if (miscCounter & 0x8)
				{
					register unsigned long temp1, temp2;
					temp1 = ((unsigned long *) srcPtr)[0];
					temp2 = ((unsigned long *) srcPtr)[1];
					((unsigned long *) destPtr)[0] = temp1;
					((unsigned long *) destPtr)[1] = temp2;
					srcPtr += 8;
					destPtr += 8;
				}
				if (miscCounter & 0x4)
				{
					register unsigned long temp1;
					temp1 = *((unsigned long *) srcPtr);
					srcPtr +=4;
					*((unsigned long *) destPtr)  = temp1;
					destPtr +=4;
				}
				if (miscCounter & 0x2)
				{
					register unsigned short temp1;
					temp1 = *((unsigned short *) srcPtr);
					srcPtr +=2;
					*((unsigned short *) destPtr)  = temp1;
					destPtr +=2;
				}
				if (miscCounter & 0x1)
					*destPtr++ = *srcPtr++;
		
				}
				
				// adjust for the padding
				srcPtr = (unsigned char *) ALIGN_TO_NEXT_LONG(srcPtr);	
				break;
						
			case kSkipPixelsToken:
				destPtr += tokenData;
				break;
						
			case kLineStartToken:
				// set up the destination pointer
				destPtr = rowStart;
				rowStart += gRowBytes;
				break;
						
			case kEndShapeToken:
				// exit the routine
				return;
				break;
#if qDebugging
			default:
				// we should never get here
				Debugger();
				break;
#endif
		}
	}	while( true );
}


/****************************************************************************************************
	TGraphic::BackgroundClipped
	
	The Background routines work similarly to the standard graphic drawing routines, but we keep an additional
	pointer to the destination graphics world, and we draw from it, not the graphic data.  Later, we might
	optimize this routine further as we know the two are always in sync.
****************************************************************************************************/


void
TGraphic::BackgroundClipped (SInt32 top, SInt32 left, UInt32 clipLeft, UInt32 clipRight, UInt32 clipTop, UInt32 clipBottom)
{
	unsigned char *sourceRowStart;			// the pointer to the start of this row in the source
	unsigned char *destRowStart;			// the pointer to the start of this row in the dest
	unsigned char *srcPtr;					// the current position in the source pixmap
	unsigned char *destPtr;					// the current position in the destination pixmap
	unsigned char *dataPtr;					// the current position in the sprite data;
	unsigned long miscCounter;				// a counter for various purposes
	unsigned long extraCounter;				// a counter for right clippling purposes ( how much extra was there? )
	unsigned long tokenOp;					// the op code from the token
	unsigned long tokenData;				// the data from the token
	unsigned long yCount;					// how many lines do we still have to draw
	unsigned long xCount;					// where are we in this line?

	// determine characteristics about the pixmap
	sourceRowStart = gBackBaseAddr + (top + clipTop) * gRowBytes + left;
	destRowStart   = gDestBaseAddr + (top + clipTop) * gRowBytes + left;
	
	// move to the start of the image data.  We'll do a quick skip of any initial pixel rows that we don't
	// need to worry about.
	dataPtr = ( unsigned char * )( *fImage);
	
	yCount = clipBottom - clipTop;
	
	while (clipTop > 0)
	{
		clipTop--;
		tokenData = ( *( ( unsigned long * )dataPtr ) ) & 0x00ffffff;
		dataPtr += sizeof( unsigned long ) + tokenData;
	}
	
	do
	{
		// get a token
		tokenOp = ( *( ( unsigned long * )dataPtr ) ) >> 24;
		tokenData = ( *( ( unsigned long * )dataPtr ) ) & 0x00ffffff;
		dataPtr += sizeof( unsigned long );
			
		// depending on the token
		switch( tokenOp )
		{
			case kDrawPixelsToken:
				miscCounter = tokenData;
				extraCounter = 0;
						
				// advance the data pointer for free
				tokenData = ALIGN_TO_NEXT_LONG (tokenData);
				dataPtr += tokenData;
						
				// if we need to, clip to the left
				if (xCount < clipLeft )
				{
					// if this run does not appear at all, don't draw it
					if ( miscCounter < clipLeft - xCount )
					{
						destPtr += miscCounter;
						srcPtr += miscCounter;
						xCount += miscCounter;
						break;
					}
					else
					{
					// if it does, skip to where we can draw
						miscCounter -= clipLeft - xCount;
						destPtr += clipLeft - xCount;
						srcPtr += clipLeft - xCount;
						xCount = clipLeft;
					}
				}
						
				// if we need to, clip to the right
				if ( xCount + miscCounter > clipRight )
				{
					// if this run does not appear at all, skip it
					if ( xCount > clipRight )
						// once xCount is greater than xRight, no drawing commands will ever
						// be issued on that line, so I changed the "gurus" code, which was actually
						// wasting a couple of instructions setting variables that aren't needed.
						//destPtr += miscCounter;
						//srcPtr += miscCounter;
						//xCount += miscCounter;
						break;
					else
					{
						// if it does, setup to draw what we can
						extraCounter = (xCount + miscCounter) - clipRight;
						miscCounter -= extraCounter;
					}
				}
						
				// adjust xCount for the run
				xCount += miscCounter;
				// This blit loop is optimized to move 32 bytes or less.  On 604 machines, using an 8 byte move
				// is faster in those rare cases where you can get them to align, and way slower when they don't.
				// because of the rarity of these cases, this blit loop doesn't do anything fancy with doubles.
				
				
				{
				register unsigned long sixteenblits, blitloop;
				sixteenblits = miscCounter >> 4;
				for ( blitloop = 0; blitloop < sixteenblits; blitloop++)
				{
						register unsigned long temp1, temp2, temp3, temp4;
						temp1 = ((unsigned long *) srcPtr)[0];
						temp2 = ((unsigned long *) srcPtr)[1];
						temp3 = ((unsigned long *) srcPtr)[2];
						temp4 = ((unsigned long *) srcPtr)[3];
						((unsigned long *) destPtr)[0] = temp1;
						((unsigned long *) destPtr)[1] = temp2;
						((unsigned long *) destPtr)[2] = temp3;
						((unsigned long *) destPtr)[3] = temp4;
						srcPtr += 16;
						destPtr += 16;
				}
				// move any remaining data, up to 15 bytes total
				if (miscCounter & 0x8)
				{
					register unsigned long temp1, temp2;
					temp1 = ((unsigned long *) srcPtr)[0];
					temp2 = ((unsigned long *) srcPtr)[1];
					((unsigned long *) destPtr)[0] = temp1;
					((unsigned long *) destPtr)[1] = temp2;
					srcPtr += 8;
					destPtr += 8;
				}
				if (miscCounter & 0x4)
				{
					register unsigned long temp1;
					temp1 = *((unsigned long *) srcPtr);
					srcPtr +=4;
					*((unsigned long *) destPtr)  = temp1;
					destPtr +=4;
				}
				if (miscCounter & 0x2)
				{
					register unsigned short temp1;
					temp1 = *((unsigned short *) srcPtr);
					srcPtr +=2;
					*((unsigned short *) destPtr)  = temp1;
					destPtr +=2;
				}
				if (miscCounter & 0x1)
					*destPtr++ = *srcPtr++;
		
				}
							
				// adjust for right clipping
				destPtr += extraCounter;
				srcPtr += extraCounter;
				xCount += extraCounter;
						
				break;
						
			case kSkipPixelsToken:
				destPtr += tokenData;
				srcPtr += tokenData;
						
				xCount += tokenData;
				break;
						
			case kLineStartToken:
				// check to see if we're done drawing
				if (yCount == 0)
					return;
				else
				{
					// set all the counters up
					yCount--;
					xCount = 0;
					// set up the source and destination pointers
					destPtr = destRowStart;
					srcPtr = sourceRowStart;
					destRowStart += gRowBytes;
					sourceRowStart += gRowBytes;
				}
				break;
			case kEndShapeToken:
				return;
				break;
#if qDebugging
			default:
			// we should never get here
			Debugger();
			break;
#endif
		}
	} while (true);
}

/****************************************************************************************************
	TGraphic::BackgroundUnclipped
	
	The Background routines work similarly to the standard graphic drawing routines, but we keep an additional
	pointer to the destination graphics world, and we draw from it, not the graphic data.  Later, we might
	optimize this routine further as we know the two are always in sync.
****************************************************************************************************/

void 
TGraphic::BackgroundUnclipped (SInt32 top, SInt32 left)
{
	unsigned char *destRowStart;				// the pointer to the start of this row
	unsigned char *sourceRowStart;
	unsigned char *srcPtr;						// the current position in the source pixmap
	unsigned char *destPtr;						// the current position in the destination pixmap
	unsigned char *dataPtr;						// the current position in the sprite data
	unsigned long miscCounter;					// a counter for various purposes
	unsigned long tokenOp;						// the op code from the token
	unsigned long tokenData;					// the data from the token
	
	// determine characteristics about the pixmap
	destRowStart   = gDestBaseAddr + top * gRowBytes + left;
	sourceRowStart = gBackBaseAddr + top * gRowBytes + left;
	
	// move to the start of the graphic data
	dataPtr = ( unsigned char * )( *fImage );
	
	// loop until we are done
	do
	{
		// get a token
		tokenOp = ( *( ( unsigned long * )dataPtr ) ) >> 24;
		tokenData = ( *( ( unsigned long * )dataPtr ) ) & 0x00ffffff;
		dataPtr += sizeof( unsigned long );
			
		// depending on the token
		switch( tokenOp )
		{
		case kDrawPixelsToken:
			miscCounter = tokenData;
			// adjust the data pointer
			tokenData = ALIGN_TO_NEXT_LONG (tokenData);
			dataPtr += tokenData;
	
				// This blit loop is optimized to move 32 bytes or less.  On 604 machines, using an 8 byte move
				// is faster in those rare cases where you can get them to align, and way slower when they don't.
				// because of the rarity of these cases, this blit loop doesn't do anything fancy with doubles.
				
				
				{
				register unsigned long sixteenblits, blitloop;
				sixteenblits = miscCounter >> 4;
				for ( blitloop = 0; blitloop < sixteenblits; blitloop++)
				{
						register unsigned long temp1, temp2, temp3, temp4;
						temp1 = ((unsigned long *) srcPtr)[0];
						temp2 = ((unsigned long *) srcPtr)[1];
						temp3 = ((unsigned long *) srcPtr)[2];
						temp4 = ((unsigned long *) srcPtr)[3];
						((unsigned long *) destPtr)[0] = temp1;
						((unsigned long *) destPtr)[1] = temp2;
						((unsigned long *) destPtr)[2] = temp3;
						((unsigned long *) destPtr)[3] = temp4;
						srcPtr += 16;
						destPtr += 16;
				}
				// move any remaining data, up to 15 bytes total
				if (miscCounter & 0x8)
				{
					register unsigned long temp1, temp2;
					temp1 = ((unsigned long *) srcPtr)[0];
					temp2 = ((unsigned long *) srcPtr)[1];
					((unsigned long *) destPtr)[0] = temp1;
					((unsigned long *) destPtr)[1] = temp2;
					srcPtr += 8;
					destPtr += 8;
				}
				if (miscCounter & 0x4)
				{
					register unsigned long temp1;
					temp1 = *((unsigned long *) srcPtr);
					srcPtr +=4;
					*((unsigned long *) destPtr)  = temp1;
					destPtr +=4;
				}
				if (miscCounter & 0x2)
				{
					register unsigned short temp1;
					temp1 = *((unsigned short *) srcPtr);
					srcPtr +=2;
					*((unsigned short *) destPtr)  = temp1;
					destPtr +=2;
				}
				if (miscCounter & 0x1)
					*destPtr++ = *srcPtr++;
		
				}
	
				break;
						
			case kSkipPixelsToken:
				destPtr += tokenData;
				srcPtr += tokenData;	
				break;
					
			case kLineStartToken:
				// set up the source and destination
				destPtr = destRowStart;
				srcPtr = sourceRowStart;
				destRowStart += gRowBytes;
				sourceRowStart += gRowBytes;
				break;
						
			case kEndShapeToken:
				// signal a loop exit
				return;
				break;
#ifdef qDebugging
			default:
				// we should never get here
				Debugger();
				break;
#endif
		}
	} while (true);
}


/****************************************************************************************************
	TGraphic::DrawMaskUnclipped
	
	This version of the drawing loop draws black anywhere the draw mask is active.  Useful for
	writing a PICT out to disk with the mask information.
****************************************************************************************************/

void 
TGraphic::DrawMaskUnclipped (SInt32 top, SInt32 left)
{
	unsigned char *destRowStart;				// the pointer to the start of this row
	unsigned char *destPtr;						// the current position in the destination pixmap
	unsigned char *dataPtr;						// the current position in the sprite data
	unsigned long miscCounter;					// a counter for various purposes
	unsigned long tokenOp;						// the op code from the token
	unsigned long tokenData;					// the data from the token
	unsigned long indexSmear;
	unsigned long indexSmear2;
	
	indexSmear = kMaskColorIndex;
	indexSmear |= (kMaskColorIndex << 8);
	indexSmear |= (indexSmear << 16);
	
	indexSmear2 = indexSmear;
	
	// determine characteristics about the pixmap
	destRowStart = gDestBaseAddr + top * gRowBytes + left;
	
	// move to the start of the graphic data
	dataPtr = ( unsigned char * )( *fImage );
	
	// loop until we are done
	do
	{
		// get a token
		tokenOp = ( *( ( unsigned long * )dataPtr ) ) >> 24;
		tokenData = ( *( ( unsigned long * )dataPtr ) ) & 0x00ffffff;
		dataPtr += sizeof( unsigned long );
			
		// depending on the token
		switch( tokenOp )
		{
		case kDrawPixelsToken:
			miscCounter = tokenData;
			// adjust the data pointer
			tokenData = ALIGN_TO_NEXT_LONG (tokenData);
			dataPtr += tokenData;
				// This blit loop is optimized to move 32 bytes or less.  On 604 machines, using an 8 byte move
				// is faster in those rare cases where you can get them to align, and way slower when they don't.
				// because of the rarity of these cases, this blit loop doesn't do anything fancy with doubles.
				
				
				{
				register unsigned long sixteenblits, blitloop;
				sixteenblits = miscCounter >> 4;
				for (blitloop = 0; blitloop < sixteenblits; blitloop++)
				{
					((unsigned long *) destPtr)[0] = *((unsigned long *) &indexSmear);
					((unsigned long *) destPtr)[1] = *((unsigned long *) &indexSmear);
					((unsigned long *) destPtr)[2] = *((unsigned long *) &indexSmear);
					((unsigned long *) destPtr)[3] = *((unsigned long *) &indexSmear);
					destPtr += 16;
				}
				// move any remaining data, up to 15 bytes total
				if (miscCounter & 0x8)
				{
					((unsigned long *) destPtr)[0] = *((unsigned long *) &indexSmear);
					((unsigned long *) destPtr)[1] = *((unsigned long *) &indexSmear);
					destPtr += 8;
				}
				if (miscCounter & 0x4)
				{
					*((unsigned long *) destPtr)  = *((unsigned long *) &indexSmear);
					destPtr +=4;
				}
				if (miscCounter & 0x2)
				{
					*((unsigned short *) destPtr) = *((unsigned short *) &indexSmear);
					destPtr +=2;
				}
				if (miscCounter & 0x1)
					*destPtr++ = *((unsigned char *) &indexSmear);
		
				}
	
				break;
						
			case kSkipPixelsToken:
				destPtr += tokenData;
				break;
					
			case kLineStartToken:
				// set up the source and destination
				destPtr = destRowStart;
				destRowStart += gRowBytes;
				break;
						
			case kEndShapeToken:
				// signal a loop exit
				return;
				break;
#ifdef qDebugging
			default:
				// we should never get here
				Debugger();
				break;
#endif
		}
	} while (true);
}

/****************************************************************************************************
	TGraphic::HitMaskUnclipped
	
	Same as DrawMaskUnclipped, but using the fHitMask data instead.
****************************************************************************************************/

void 
TGraphic::HitMaskUnclipped (SInt32 top, SInt32 left)
{
	unsigned char *destRowStart;				// the pointer to the start of this row
	unsigned char *destPtr;						// the current position in the destination pixmap
	unsigned char *dataPtr;						// the current position in the sprite data
	unsigned long miscCounter;					// a counter for various purposes
	unsigned long tokenOp;						// the op code from the token
	unsigned long tokenData;					// the data from the token
	unsigned long indexSmear;
	unsigned long indexSmear2;
	
	indexSmear = kMaskColorIndex;
	indexSmear |= (kMaskColorIndex << 8);
	indexSmear |= (indexSmear << 16);
	
	indexSmear2 = indexSmear;
	
	// determine characteristics about the pixmap
	destRowStart = gDestBaseAddr + top * gRowBytes + left;
	
	// move to the start of the graphic data
	dataPtr = ( unsigned char * )( *fHitMask );
	
	// loop until we are done
	do
	{
		// get a token
		tokenOp = ( *( ( unsigned long * )dataPtr ) ) >> 24;
		tokenData = ( *( ( unsigned long * )dataPtr ) ) & 0x00ffffff;
		dataPtr += sizeof( unsigned long );
			
		// depending on the token
		switch( tokenOp )
		{
		case kDrawPixelsToken:
			miscCounter = tokenData;
	
				// This blit loop is optimized to move 32 bytes or less.  On 604 machines, using an 8 byte move
				// is faster in those rare cases where you can get them to align, and way slower when they don't.
				// because of the rarity of these cases, this blit loop doesn't do anything fancy with doubles.
				
				
				{
				register unsigned long sixteenblits;
				sixteenblits = miscCounter >> 4;
				if (sixteenblits)
				{
					do
					{
						((unsigned long *) destPtr)[0] = *((unsigned long *) &indexSmear);
						((unsigned long *) destPtr)[1] = *((unsigned long *) &indexSmear);
						((unsigned long *) destPtr)[2] = *((unsigned long *) &indexSmear);
						((unsigned long *) destPtr)[3] = *((unsigned long *) &indexSmear);
						destPtr += 16;
					}  while (--sixteenblits != 0);
				}
				// move any remaining data, up to 15 bytes total
				if (miscCounter & 0x8)
				{
					((unsigned long *) destPtr)[0] = *((unsigned long *) &indexSmear);
					((unsigned long *) destPtr)[1] = *((unsigned long *) &indexSmear);
					destPtr += 8;
				}
				if (miscCounter & 0x4)
				{
					*((unsigned long *) destPtr)  = *((unsigned long *) &indexSmear);
					destPtr +=4;
				}
				if (miscCounter & 0x2)
				{
					*((unsigned short *) destPtr) = *((unsigned short *) &indexSmear);
					destPtr +=2;
				}
				if (miscCounter & 0x1)
					*destPtr++ = *((unsigned char *) &indexSmear);
		
				}

				break;
						
			case kSkipPixelsToken:
				destPtr += tokenData;
				break;
					
			case kLineStartToken:
				// set up the source and destination
				destPtr = destRowStart;
				destRowStart += gRowBytes;
				break;
						
			case kEndShapeToken:
				// signal a loop exit
				return;
				break;
#ifdef qDebugging
			default:
				// we should never get here
				Debugger();
				break;
#endif
		}
	} while (true);
}

